#! /usr/local/bin/python

from Tkinter import *
import os
import string 
import sys

def normalize_pathname(path):
    """Given PATHNAME, return an absolute pathname relative to cwd, reducing
    out all current-dir (unix: '.') and parent-dir (unix: '..') references."""

    outwards, inwards = 0, []
    for nm in string.splitfields(path, os.sep):
	if nm != os.curdir:
	    if nm == os.pardir:
		# Translate parent-dir entries to outward notches:
		if inwards:
		    # Pop a containing inwards:
		    del inwards[-1]
		else:
		    # Register leading outward notches:
		    outwards = outwards + 1
	    else:
		inwards.append(nm)
    inwards = string.joinfields(inwards, os.sep)

    if (not inwards) or (inwards[0] != os.sep):
	# Relative path - join with current working directory, (ascending
	# outwards to account for leading parent-dir components):
	cwd = os.getcwd()
	if outwards:
	    cwd = string.splitfields(cwd, os.sep)
	    cwd = string.joinfields(cwd[:len(cwd) - outwards], os.sep)
	if inwards:
	    return os.path.join(cwd, inwards)				# ==>
	else:
	    return cwd							# ==>
    else:
	return inwards							# ==>

def newdialog(master, text):
	"""Given a text, presents a dialog w/ a title containing that text, an entry,
	an OK and cancel button.  The OK button is the default.

	Returns the text in the entry widget, or None if canceled."""

	OKBUTTON = 0
	CANCELBUTTON = 1
	w = Toplevel( master)# class = text)
	w.title( text )
	w.iconname( text )
	top = Frame( w, relief = GROOVE, bd = 1)
	bot = Frame( w, relief = GROOVE, bd = 1)
	msg = Message( top, width = '3i', text = text, font = '-Adobe-Times-Medium-R-Normal-*-180-*')
	msg.pack( fill = X, padx = '2m', pady = '2m')
	top.pack( expand = 1, side = TOP, fill = BOTH)
	entry = Entry( top, relief = GROOVE)
	entry.pack( pady = '2m', anchor = CENTER)

	ok = Button( bot, relief = RIDGE, borderwidth = 3, text = 'OK', command = ('set', 'button', OKBUTTON))
	ok.pack ( side = LEFT, expand = 1, padx = 0, pady = 0, ipady = 0, fill = X)
	cancel = Button(bot, text = 'Cancel', relief = GROOVE, command = ('set', 'button', CANCELBUTTON))
	cancel.pack (side = LEFT, expand = 1, padx = 0, pady = 0, ipady = 0, fill = X)
	bot.pack(side = BOTTOM, fill = BOTH)

	default =0
	w.bind('<Return>',
	       lambda e, b=ok, i=OKBUTTON:
	       (b.flash(),
		b.setvar('button', i)))
	oldFocus = w.tk.call('focus') # XXX
	w.grab_set()
	w.focus()
	entry.focus()
	dirname = None
	w.waitvar('button')
	retu = w.getint(w.getvar('button'))
	if retu == OKBUTTON:
		dirname = entry.get()
		try:
			os.mkdir(dirname, 755 )
		except os.error:
			pass
	w.destroy()
	w.tk.call('focus', oldFocus) # XXX
	return dirname

def fileselector(new, master, title, text = None, origdir = None):
	"""Given a title, and a text, and optionally a starting directory (defaults to
	current directory), presents a MODAL file selection dialog box which allows navigation
	up and down the file structure.   Allows the creation of new subdirectories and the
	naming of a file which does not currently exist.  Does not create such files.

	Returns the full, normalized pathname to the selected file, or None if the dialog
	is canceled"""

	NEWBUTTON = 0
	OPENBUTTON = 1
	CANCELBUTTON = 2
	UPBUTTON = 3
	ORIGBUTTON = 4
	NEWDIRBUTTON = 5

	if origdir == None:
		origdir = os.curdir
	origdir = normalize_pathname(origdir)

	# 1. Create the top-level window and divide it into top
	# and bottom parts.

	w = Toplevel(master) #, class = 'Dialog')
	w.title(title)
	w.iconname('Dialog')

	if text:
		top = Frame( w, relief = GROOVE, bd = 1)
		msg = Message( top, width = '2.5i', text = text, font = '-Adobe-Times-Medium-R-Normal-*-180-*')
		msg.pack( expand = 1, fill = X, padx = '2m', pady = '2m')
		top.pack( side = TOP, fill = BOTH)

	middle = Frame( w, relief = GROOVE, bd = 1)
	middle.pack( fill = BOTH, expand = 1)
	left = Frame( middle, bd = 1)
	left.pack( side = LEFT, fill = BOTH, expand = 1)
	right = Frame( middle, bd = 1)
	right.pack( side = RIGHT, fill = BOTH, expand = 1)
	label = Label( left, text = normalize_pathname(os.curdir))
	label.pack( fill = X, anchor = W)

	f = Frame( left, relief = SUNKEN, borderwidth = 1)
	filelist = Listbox( f, highlightcolor = 'grey', selectmode = BROWSE, selectborderwidth = 0,
			    width = 25, relief = FLAT, height = 10, borderwidth = 0)

	objectscroll = Scrollbar(left, highlightcolor = 'grey', orient = VERTICAL, width = 12, borderwidth = 1)
	filelist['yscrollcommand'] = objectscroll.set
	objectscroll['command'] = filelist.yview
	filelist.pack( expand = 1, fill = BOTH)
	f.pack({'side':'left','expand':1,'fill':'both', 'pady':2})
	objectscroll.pack({'side':'right','fill':'both'})
	
	up = Button(right, text = 'Up', relief = GROOVE, command = ('set','button',UPBUTTON))
	up.pack(expand = 1, padx = 0, pady = 0, ipady = 0, fill = X)
	orig = Button(right, text = 'Original Directory', relief = GROOVE, command = ('set','button',ORIGBUTTON))
	orig.pack(expand = 1, padx = 0, pady = 0, ipady = 0, fill = X)
	if new:
		newdir = Button(right, text = 'New Directory...', relief = GROOVE,
				command = ('set','button',NEWDIRBUTTON))
		newdir.pack(expand = 1, padx = 0, pady = 0, ipady = 0, fill = X)
		new = Button(right, text = 'New File...', relief = GROOVE,
			     command = ('set', 'button', NEWBUTTON))
		new.pack (expand = 1, padx = 0, pady = 0, ipady = 0, fill = X)
	open = Button(right, text = 'Open', borderwidth = 3, relief = RIDGE,
		      command = ('set', 'button', OPENBUTTON))
	open.pack (expand = 1, padx = 0, pady = 0, ipady = 0, fill = X)
	cancel = Button(right, text = 'Cancel', relief = GROOVE, command = ('set', 'button', CANCELBUTTON))
	cancel.pack ( expand = 1, padx = 0, pady = 0, ipady = 0, fill = X)
	# 4. Set up a binding for <Return>, if there's a default,
	# set a grab, and claim the focus too.

	default =1
	w.bind('<Return>',
	       lambda e, b=open, i=OPENBUTTON:
	       (b.flash(),
		b.setvar('button', i)))
	filelist.bind('<Double-1>',
	       lambda e, b=open, i=OPENBUTTON:
	       (b.flash(),
		b.setvar('button', i)))
	filelist.bind('<Escape>',
	       lambda e, b=cancel, i = CANCELBUTTON:
	       (b.flash(),
		b.setvar('button', i)))
	filelist.bind('<Left>',
	       lambda e, b=up, i = UPBUTTON:
	       (b.flash(),
		b.setvar('button', i)))
	filelist.bind('<Right>',
	       lambda e, b=open, i = OPENBUTTON:
	       (b.flash(),
		b.setvar('button', i)))
	oldFocus = w.tk.call('focus') # XXX
	w.grab_set()
	filelist.focus()

	# 5. Wait for the user to respond, then restore the focus
	# and return the index of the selected button.

	filename = None
	retu = None
	while (filename == None and retu != CANCELBUTTON):
		filelist.delete(0,'end')
		try:
			l = os.listdir(os.curdir)
			l.sort()
			for file in l:
				if file != os.curdir and file != os.pardir:
					if (os.path.isdir(file)):
						filelist.insert('end',file+os.sep)
					else:
						filelist.insert('end',file)
		except os.error:
			pass
		dname =  normalize_pathname(os.curdir)
		if len(dname) > 25:
			dname = dname[:10]+'...'+dname[-15:]
		label['text'] = dname

		filelist.select_set(0,0)
		w.waitvar('button')
		retu = w.getint(w.getvar('button'))
		if retu == NEWBUTTON:   # New File
			filename = newdialog(master, 'Create file named:')
		elif retu == UPBUTTON:
			try:
				os.chdir(os.pardir)
			except:
				pass
		elif retu == ORIGBUTTON:
			os.chdir(origdir)
		elif retu == NEWDIRBUTTON:
			dir = newdialog(master, 'Create directory named:')
			if dir:
				try:
					os.chdir(dir)
				except os.error:
					pass
		elif retu == OPENBUTTON:
			selection = filelist.curselection()
			if selection and len(selection) == 1:
				f = filelist.get(selection[0])
				try:
					if (os.path.isdir(f)):
						os.chdir(filelist.get(selection[0]))
					elif new == 0:
						filename = f
				except os.error:
					pass

	if retu == CANCELBUTTON:
		os.chdir(origdir)

	w.destroy()
	w.tk.call('focus', oldFocus) # XXX
	if filename:
		return normalize_pathname(filename)
	else:
		return None

def newfileselector(master, title, text = None, origdir = None):
	return fileselector(1, master, title, text, origdir)

def oldfileselector(master, title, text = None, origdir = None):
	return fileselector(0, master, title, text, origdir)


def go():
	i = newfileselector(mainWidget, 'Subject Set Selector')
	if i:
		print i

def test():
	import sys
	global mainWidget
	mainWidget = Frame()
	Pack.config( mainWidget )
	start = Button( mainWidget, text= 'Press Here To Start', command = go )
	start.pack( fill = BOTH )
	endit = Button( mainWidget, text = 'Exit', command = sys.exit )
	endit.pack( fill = BOTH )
	mainWidget.mainloop()
	
if __name__ == '__main__':
	test()
